"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CborEncoder = void 0;
const isFloat32_1 = require("../../util/buffers/isFloat32");
const CborEncoderFast_1 = require("./CborEncoderFast");
const isSafeInteger = Number.isSafeInteger;
class CborEncoder extends CborEncoderFast_1.CborEncoderFast {
    writeAny(value) {
        switch (typeof value) {
            case 'number':
                return this.writeNumber(value);
            case 'string':
                return this.writeStr(value);
            case 'boolean':
                return this.writer.u8(0xf4 + +value);
            case 'object': {
                if (!value)
                    return this.writer.u8(0xf6);
                const constructor = value.constructor;
                switch (constructor) {
                    case Array:
                        return this.writeArr(value);
                    case Uint8Array:
                        return this.writeBin(value);
                    case Map:
                        return this.writeMap(value);
                    default:
                        return this.writeObj(value);
                }
            }
            case 'undefined':
                return this.writer.u8(0xf7);
            case 'bigint':
                return this.writeBigInt(value);
        }
    }
    writeNumber(num) {
        if (isSafeInteger(num))
            this.writeInteger(num);
        else if (typeof num === 'bigint')
            this.writeBigInt(num);
        else
            this.writeFloat(num);
    }
    writeBigInt(int) {
        if (int >= 0)
            this.writeBigUint(int);
        else
            this.writeBigSint(int);
    }
    writeBigUint(uint) {
        switch (true) {
            case uint <= 23:
                this.writer.u8(0 + Number(uint));
                break;
            case uint <= 0xff:
                this.writer.u16((0x18 << 8) + Number(uint));
                break;
            case uint <= 0xffff:
                this.writer.u8u16(0x19, Number(uint));
                break;
            case uint <= 0xffffffff:
                this.writer.u8u32(0x1a, Number(uint));
                break;
            default: {
                this.writer.u8u64(0x1b, uint);
                break;
            }
        }
    }
    writeBigSint(int) {
        const uint = -BigInt(1) - int;
        switch (true) {
            case uint <= 24:
                this.writer.u8(32 + Number(uint));
                break;
            case uint <= 0xff:
                this.writer.u16((0x38 << 8) + Number(uint));
                break;
            case uint <= 0xffff:
                this.writer.u8u16(0x39, Number(uint));
                break;
            case uint <= 0xffffffff:
                this.writer.u8u32(0x3a, Number(uint));
                break;
            default:
                this.writer.u8u64(0x3b, uint);
                break;
        }
    }
    writeFloat(float) {
        if ((0, isFloat32_1.isFloat32)(float))
            this.writer.u8f32(0xfa, float);
        else
            this.writer.u8f64(0xfb, float);
    }
    writeMap(map) {
        this.writeMapHdr(map.size);
        map.forEach((value, key) => {
            this.writeAny(key);
            this.writeAny(value);
        });
    }
}
exports.CborEncoder = CborEncoder;
